import tkinter as tk
from tkinter import ttk

import pandas as pd
from PIL import Image, ImageTk
from threading import Thread
import seaborn as sns
from matplotlib.backends.backend_tkagg import FigureCanvasTkAgg
from matplotlib.figure import Figure

from scraper import Facade


class GUI(tk.Frame):
    def __init__(self):
        super().__init__(root)
        self.__facade = Facade()
        self.__search_results_list = []
        self.search_thread = None
        self.__search_results_df = None
        self.select_thread = None
        self.__selected_book_list = []
        self.__name_bookobj_dict = {}


        # Root related section
        self.__root = root
        self.__root.title("Footnote")
        self.__root.geometry("1080x700")
        self.__root.resizable(width=False, height=False)  # This line prevents the ability to resize the root window.

        # Background Visual section
        self.__background_image = ImageTk.PhotoImage(Image.open('images/footnote_images/bg.jpg'))  # This line assigns the background image to self.background_image
        self.__background_label = tk.Label(self.__root, image=self.__background_image)
        self.__background_label.place(x=0, y=0)

        # Main main_frame
        self.__main_frame = tk.Frame(self.__root, width=1000, height=525)
        self.__main_frame.place(x=40, y=247.5)

        # Search frame section
        self.__search_frame = tk.Frame(self.__main_frame)
        self.__sf_canvas = tk.Canvas(self.__search_frame, width=350 * 0.75, height=420)

        self.__sf_intro_subtitle = tk.Label(self.__search_frame, text="First,", font=("Arial", 10, "bold"), fg="#002454")
        self.__sf_title = tk.Label(self.__search_frame, text="Find a book", font=("Arial", 20, "bold"), fg="#002454")
        self.__sf_subtitle = tk.Label(self.__search_frame, text="Which would you like to search from?", font=("Arial", 10, "bold"), fg="#002454")

        self.__rb_frame = tk.Frame(self.__search_frame)
        self.__store_select = tk.StringVar()
        rb_meb = tk.Radiobutton(self.__rb_frame, text="Meb", value="meb", variable=self.__store_select)
        rb_nai = tk.Radiobutton(self.__rb_frame, text="Naiin", value="naiin", variable=self.__store_select)
        rb_ook = tk.Radiobutton(self.__rb_frame, text="Ookbee", value="ookbee", variable=self.__store_select)
        rb_meb.pack(side=tk.LEFT)
        rb_nai.pack(side=tk.LEFT)
        rb_ook.pack(side=tk.LEFT)

        # Entry for the search keyword
        self.__search_keyword = tk.StringVar()
        self.__search_entry = tk.Entry(self.__search_frame, width=30, textvariable=self.__search_keyword)

        # Button for the search
        self.__search_button = tk.Button(self.__search_frame, text="Search", width=10, height=1, bg="#002454", fg='white',
                                         command=lambda: self.run_search(self.__store_select, self.__search_keyword))

        # Progress bar - The styling section was adapted from the ChatGPT generated code.
        self.__progress_bar_1 = ttk.Progressbar(self.__search_frame, length=245, mode="indeterminate", style="Custom.Horizontal.TProgressbar")
        self.__progress_bar_1.configure(style="Custom.Horizontal.TProgressbar")
        self.__progress_bar_1["style"] = "Custom.Horizontal.TProgressbar"
        self.style = ttk.Style()
        self.style.theme_use('clam')
        self.style.configure("Custom.Horizontal.TProgressbar", troughcolor="#D9D9D9", bordercolor="#D9D9D9", background="#002454")

        self.__progress_label = tk.Label(self.__search_frame, text="")

        # Search results section
        self.__search_results_label = tk.Label(self.__search_frame, text="Search Results:", font=("Arial", 10, "bold"), fg="#002454")

        # The listbox for the search results
        self.__search_result_listbox = tk.Listbox(self.__search_frame, width=30, height=5)
        self.__search_result_listbox_yscrollbar = tk.Scrollbar(self.__search_frame, orient=tk.VERTICAL, command=self.__search_result_listbox.yview)
        self.__search_result_listbox.config(yscrollcommand=self.__search_result_listbox_yscrollbar.set)
        self.__search_result_listbox_xscrollbar = tk.Scrollbar(self.__search_frame, orient=tk.HORIZONTAL, command=self.__search_result_listbox.xview)
        self.__search_result_listbox.config(yscrollcommand=self.__search_result_listbox_xscrollbar.set)

        # Button for the selection
        self.__select_button = tk.Button(self.__search_frame, text="Select", width=10, height=1, bg="#002454", fg='white',
                                         command=lambda: self.select())

        self.__selecting_variable = tk.StringVar()
        self.__selecting_label = tk.Label(self.__search_frame, textvariable=self.__selecting_variable)

        # Selection frame section
        self.__selection_frame = tk.Frame(self.__main_frame)
        self.__sef_canvas = tk.Canvas(self.__selection_frame, width=350 * 0.75, height=420)

        self.__sef_intro_subtitle = tk.Label(self.__selection_frame, text="Second,", font=("Arial", 10, "bold"), fg="#002454")
        self.__sef_title = tk.Label(self.__selection_frame, text="Selection", font=("Arial", 20, "bold"), fg="#002454")
        self.__sef_subtitle = tk.Label(self.__selection_frame, text="Here's your selections:", font=("Arial", 10, "bold"), fg="#002454")

        self.__selection_listbox = tk.Listbox(self.__selection_frame, width=30, height=16)
        self.__selection_listbox_yscrollbar = tk.Scrollbar(self.__selection_frame, orient=tk.VERTICAL, command=self.__selection_listbox.yview)
        self.__selection_listbox.config(yscrollcommand=self.__selection_listbox_yscrollbar.set)
        self.__selection_listbox_xscrollbar = tk.Scrollbar(self.__selection_frame, orient=tk.HORIZONTAL, command=self.__selection_listbox.xview)
        self.__selection_listbox.config(xscrollcommand=self.__selection_listbox_xscrollbar.set)

        self.__clear_selection_button = tk.Button(self.__selection_frame, text="Clear", width=10, height=1, bg="#002454", fg='white',
                                                  command=self.clear_selection)
        self.__compare_selection_button = tk.Button(self.__selection_frame, text="Compare", width=10, height=1, bg="#002454", fg='white',
                                                    command=self.run_selection_scraping)

        self.__compare_frame = tk.Frame(self.__main_frame)
        self.__com_canvas = tk.Canvas(self.__compare_frame, width=333, height=420)

        self.__com_intro_subtitle = tk.Label(self.__compare_frame, text="Thrid,", font=("Arial", 10, "bold"), fg="#002454")
        self.__com_title = tk.Label(self.__compare_frame, text="Enjoy", font=("Arial", 20, "bold"), fg="#002454")
        self.__com_subtitle = tk.Label(self.__compare_frame, text="Here's our conclusion:", font=("Arial", 10, "bold"), fg="#002454")

        self.__min_ex_label = tk.Label(self.__compare_frame, text="The best price we've found:", font=("Arial", 10, "bold"), fg="#002454")
        self.__title = tk.StringVar()
        self.__title_label = tk.Label(self.__compare_frame, textvariable=self.__title, font=("Arial", 10, "bold"), fg="#002454")
        self.__price =tk.StringVar()
        self.__price_label = tk.Label(self.__compare_frame, textvariable=self.__price, font=("Arial", 10, "bold"), fg="#002454")
        self.__author = tk.StringVar()
        self.__author_label = tk.Label(self.__compare_frame, textvariable=self.__author, font=("Arial", 10, "bold"), fg="#002454")
        self.__publisher = tk.StringVar()
        self.__publisher_label = tk.Label(self.__compare_frame, textvariable=self.__publisher, font=("Arial", 10, "bold"), fg="#002454")
        self.__type = tk.StringVar()
        self.__type_label = tk.Label(self.__compare_frame, textvariable=self.__type, font=("Arial", 10, "bold"), fg="#002454")
        self.__link = tk.StringVar()
        self.__link_label = tk.Label(self.__compare_frame, textvariable=self.__link, font=("Arial", 10, "bold"), fg="#002454")

        # Layout Management
        self.__search_frame.place(x=0, y=0)
        self.__selection_frame.place(x=265, y=0)
        self.__compare_frame.place(x=490, y=0)

        self.__sf_intro_subtitle.place(x=10, y=10)
        self.__sf_title.place(x=10, y=30)
        self.__sf_subtitle.place(x=10, y=70)
        self.__rb_frame.place(x=10, y=90)
        self.__search_entry.place(x=10, y=120)
        self.__search_button.place(x=10, y=150)
        self.__progress_bar_1.place(x=10, y=190)
        self.__progress_label.place(x=10, y=220)
        self.__search_results_label.place(x=10, y=250)
        self.__search_result_listbox.place(x=10, y=270)
        self.__search_result_listbox_yscrollbar.place(x=200, y=270, height=85)
        self.__search_result_listbox_xscrollbar.place(x=10, y=360, width=85)
        self.__select_button.place(x=10, y=390)
        self.__selecting_label.place(x=100, y=390)

        self.__sef_intro_subtitle.place(x=10, y=10)
        self.__sef_title.place(x=10, y=30)
        self.__sef_subtitle.place(x=10, y=70)
        self.__selection_listbox.place(x=10, y=90)
        self.__selection_listbox_yscrollbar.place(x=200, y=90, height=270)
        self.__selection_listbox_xscrollbar.place(x=10, y=360, width=85)
        self.__clear_selection_button.place(x=10, y=390)
        self.__compare_selection_button.place(x=100, y=390)

        self.__com_intro_subtitle.place(x=10, y=10)
        self.__com_title.place(x=10, y=30)
        self.__com_subtitle.place(x=10, y=70)
        self.__min_ex_label.place(x=10, y=100)
        self.__title_label.place(x=10, y=130)
        self.__price_label.place(x=10, y=160)
        self.__author_label.place(x=10, y=190)
        self.__publisher_label.place(x=10, y=220)
        self.__type_label.place(x=10, y=250)
        self.__link_label.place(x=10, y=280)

        self.__sf_canvas.pack()
        self.__sef_canvas.pack()
        self.__com_canvas.pack()

    def clear_selection(self):
        self.__selection_listbox.delete(0, tk.END)
        self.__selected_book_list = []
        self.__selection_df.drop(self.__selection_df.index, inplace=True)

    def selection_scraping(self):
        self.__selection_df = pd.DataFrame()
        print('Scraping selection...')
        books_with_info = self.__facade.scrape_selection(self.__selected_book_list)
        self.__selection_df = self.__facade.books_to_df(books_with_info)

        scp_window = tk.Toplevel(self.__root)
        scp_fig = Figure(figsize=(5, 4))
        scp_ax = scp_fig.add_subplot(111)
        sns.scatterplot(data=self.__selection_df, x='Price', y="Rating", ax=scp_ax)
        scp_ax.set_title("Scatter Plot of book price and rating")
        scp_ax.set_xlabel("Price (THB)")
        scp_ax.set_ylabel("Rating")
        scp_canvas = FigureCanvasTkAgg(scp_fig, master=scp_window)
        scp_canvas.draw()
        scp_canvas.get_tk_widget().pack(side=tk.TOP, fill=tk.BOTH, expand=1)

        min_price_book = self.__selection_df.loc[self.__selection_df['Price'].idxmin()]
        rbook = self.__facade.df_to_book(min_price_book)
        self.__title.set(rbook.name)
        self.__price.set(rbook.price)
        self.__author.set(rbook.author)
        self.__publisher.set(rbook.publisher)
        self.__type.set(rbook.book_type)
        self.__link.set(rbook.book_link)

        self.__selected_book_list = []
        self.__selection_df.drop(self.__selection_df.index, inplace=True)



    def run_selection_scraping(self):
        self.__progress_label.config(text="Scraping information... might take a while")
        self.__progress_bar_1.start()
        self.__compare_selection_button.configure(state="disabled")
        self.__selection_listbox.delete(0, tk.END)
        self.select_thread = Thread(target=self.selection_scraping)
        self.select_thread.start()
        self.__root.after(10, self.check_selection_scraping())

    def check_selection_scraping(self):
        if self.select_thread.is_alive():
            self.__root.after(10, self.check_selection_scraping)
        else:
            self.__progress_label.config(text="Done")
            self.__compare_selection_button.configure(state="normal")
            self.__progress_bar_1.stop()

    def select(self):
        selecting = self.__search_result_listbox.curselection()
        if selecting:
            book_index = selecting[0]
            observing_book = self.__name_bookobj_dict[self.__search_result_listbox.get(book_index)]
            self.__selecting_variable.set(observing_book.name)
            self.__selected_book_list += [observing_book]
            self.__search_result_listbox.delete(book_index)
            self.__selection_listbox.insert(tk.END, observing_book.name)

    def search(self, selected, search_keyword):
        self.__search_results_list = self.__facade.searching(str(selected.get()), str(search_keyword.get()))
        self.__search_results_df = self.__facade.books_to_df(self.__search_results_list)
        for search_results in self.__search_results_list:
            self.__name_bookobj_dict[search_results.name] = search_results
            self.__search_result_listbox.insert(tk.END, search_results.name)
        hist_window = tk.Toplevel(self.__root)
        fig = Figure(figsize=(5, 4))
        ax = fig.add_subplot(111)
        sns.histplot(data=self.__search_results_df, x='Price', ax=ax)
        ax.set_title("Histogram of Search Result Book Price")
        ax.set_xlabel("Price (THB)")
        ax.set_ylabel("Number of Book")
        hist_canvas = FigureCanvasTkAgg(fig, master=hist_window)
        hist_canvas.draw()
        hist_canvas.get_tk_widget().pack(side=tk.TOP, fill=tk.BOTH, expand=1)

    def run_search(self, selected, search_keyword):
        self.__progress_label.config(text="Searching...")
        self.__search_button.configure(state="disabled")
        self.__search_result_listbox.delete(0, tk.END)
        self.search_thread = Thread(target=self.search, args=(selected, search_keyword))
        self.search_thread.start()
        self.__progress_bar_1.start()
        self.__root.after(10, self.check_search)

    def check_search(self):
        if self.search_thread.is_alive():
            self.__root.after(10, self.check_search)
        else:
            self.__progress_label.config(text="Done")
            self.__search_button.configure(state="normal")
            self.__progress_bar_1.stop()


if __name__ == '__main__':
    root = tk.Tk()
    app = GUI()
    app.mainloop()
