import time
from selenium import webdriver
from selenium.common.exceptions import NoSuchElementException
from selenium.webdriver.common.by import By
from book import Book
from book import BookRelatedTools


class MebSearchScraper:
    """ Responsibility: scraping Meb """

    # Class constructor
    def __init__(self):
        self.__url = ''
        self.__keyword = ''
        self.__option = webdriver.ChromeOptions()  # create a webdriver option
        self.__option.add_argument('--headless')  # add the argument 'headless' which the webdriver interface won't appear during the scraping process
        self.__driver = webdriver.Chrome(options=self.__option)  # attach the option we've created to the webdriver

    def overview_scraping(self, keyword):
        self.__keyword = keyword.replace(' ', '%20')  # convert into the search pattern of the website
        self.__url = f'https://www.mebmarket.com/index.php?action=search_book&type=all&search={self.__keyword}&page_no=1 '  # generate the url from the search pattern
        self.__driver.get(self.__url)  # open the url we generated
        self.__driver.get(self.__url)  # meb frequently has an advertisement popup that is unable to detect by selenium.
        time.sleep(2)  # The extra .get() and the time .sleep helps refresh the driver and make sure there is a time that the element can appear in the driver.

        search_page = self.__driver.find_elements(By.CSS_SELECTOR, 'div.col_book_list')
        books = []

        for book_element in search_page:
            try:
                book_name = book_element.find_element(By.CSS_SELECTOR, '.book_name a').text
            except NoSuchElementException:
                book_name = 'No Data'
                break

            try:
                author = book_element.find_element(By.CSS_SELECTOR, '.book_meta_info a[title]').text
            except NoSuchElementException:
                author = 'No Data'

            try:
                try:
                    price = float(book_element.find_element(By.CSS_SELECTOR, 'input.button_book_list').get_attribute('price'))
                except TypeError:
                    price = 0
            except NoSuchElementException:
                price = 'No Data'

            book_link = book_element.find_element(By.CSS_SELECTOR, 'a').get_attribute('href')

            source = 'meb'

            books += [Book(name=book_name, author=author, price=price, book_link=book_link, source=source)]

        return books


class NaiinSearchScraper:
    """ Responsibility: scraping Naiin (Works like MebSearchScraper) """

    def __init__(self):
        self.__url = ''
        self.__keyword = ''
        self.__option = webdriver.ChromeOptions()
        self.__option.add_argument('--headless')
        self.__driver = webdriver.Chrome(options=self.__option)

    def overview_scraping(self, keyword):
        self.__keyword = keyword.replace(' ', '+')
        self.__url = f'https://www.naiin.com/search-result?title={self.__keyword}'
        self.__driver.get(self.__url)
        self.__driver.get(self.__url)

        search_page = self.__driver.find_elements(By.CSS_SELECTOR, 'div.productitem')
        books = []

        for book_element in search_page:
            try:
                book_name = book_element.find_element(By.CSS_SELECTOR, '.txt-normal a').text
            except NoSuchElementException:
                break

            try:
                author = book_element.find_element(By.CSS_SELECTOR, '.txt-custom-light a').text
            except NoSuchElementException:
                author = 'No Data'

            try:
                try:
                    price = float(book_element.find_element(By.CSS_SELECTOR, 'p.txt-price').text.removesuffix(' บาท'))
                except ValueError:
                    price = 0
            except NoSuchElementException:
                price = 'No Data'

            book_link = book_element.find_element(By.CSS_SELECTOR, 'a').get_attribute('href')

            source = 'naiin'

            books += [Book(name=book_name, author=author, price=price, book_link=book_link, source=source)]

        return books


class OokbeeSearchScraper:
    """ Responsibility: scraping Ookbee (Works like MebSearchScraper) """

    def __init__(self):
        self.__url = ''
        self.__keyword = ''
        self.__option = webdriver.ChromeOptions()
        self.__option.add_argument('--headless')
        self.__driver = webdriver.Chrome(options=self.__option)

    def overview_scraping(self, keyword):
        self.__keyword = keyword.replace(' ', '%20')
        self.__url = f'https://www.ookbee.com/search?keyword={self.__keyword}&searchType=all'
        self.__driver.get(self.__url)
        self.__driver.get(self.__url)

        search_page = self.__driver.find_elements(By.CSS_SELECTOR, 'div.item')
        books = []

        for book_element in search_page:
            try:
                book_name = book_element.find_element(By.CSS_SELECTOR, '.contentItemBody div').text
            except NoSuchElementException:
                break

            try:
                author = book_element.find_element(By.CSS_SELECTOR, 'p').text
            except NoSuchElementException:
                author = 'No Data'

            try:
                try:
                    price = float(book_element.find_element(By.CSS_SELECTOR, 'span').text.removesuffix(' ฿'))
                except ValueError:
                    price = 0
            except NoSuchElementException:
                price = 'No Data'

            book_link = book_element.find_element(By.CSS_SELECTOR, 'a').get_attribute('href')

            source = 'ookbee'

            books += [Book(name=book_name, author=author, price=price, book_link=book_link, source=source)]

        return books


class SelectionScraper:
    def __init__(self):
        self.__url = ''
        self.__keyword = ''
        self.__option = webdriver.ChromeOptions()
        self.__option.add_argument('--headless')
        self.__driver = webdriver.Chrome(options=self.__option)
        self.__scraped_selected_books = []

    def scrape_book(self, book_list):
        for book in book_list:
            if book.source == 'meb':
                self.__url = book.book_link
                self.__driver.get(self.__url)
                self.__driver.get(self.__url)
                time.sleep(2)

                book.name = self.__driver.find_element(By.CLASS_NAME, 'book_name').text

                book_meta_info = self.__driver.find_elements(By.CLASS_NAME, 'mete_data_text')
                book.author = book_meta_info[0].text.replace('โดย', '')
                book.publisher = book_meta_info[1].text.replace('สำนักพิมพ์', '')
                book.book_type = self.__driver.find_element(By.CSS_SELECTOR,
                                                            '#book_details_lg > div.wrap_book_details > div > div.mete_data > div.mete_data_other > div:nth-child(2) > div:nth-child(2)').text
                try:
                    book.rating = float(self.__driver.find_element(By.CSS_SELECTOR, 'div.rating_text').text)
                except ValueError:
                    book.rating = 0

                book.price = float(self.__driver.find_element(By.CSS_SELECTOR, '#buy_book_button').get_attribute('price'))
                self.__scraped_selected_books += [book]

            elif book.source == 'naiin':
                self.__url = book.book_link
                self.__driver.get(self.__url)
                self.__driver.get(self.__url)

                book.name = self.__driver.find_element(By.CLASS_NAME, 'title-topic').text
                book.author = self.__driver.find_element(By.CSS_SELECTOR,
                                                         '#main > div > div > div.bookdetail-row-1.book-detail-block.mt10.mb10 > div > div.col-xs-12.col-md-8.mt10.mb10 > div > div.row > div.col-sm-7 > p:nth-child(2) > a').text
                book.publisher = self.__driver.find_element(By.CSS_SELECTOR,
                                                            '#main > div > div > div.bookdetail-row-1.book-detail-block.mt10.mb10 > div > div.col-xs-12.col-md-8.mt10.mb10 > div > div.row > div.col-sm-7 > p:nth-child(3) > a').text
                book.book_type = self.__driver.find_element(By.CSS_SELECTOR,
                                                            '#main > div > div > div.bookdetail-row-1.book-detail-block.mt10.mb10 > div > div.col-xs-12.col-md-8.mt10.mb10 > div > div.additional-information-grid.additional-information > div:nth-child(2) > div:nth-child(3) > a').text
                book.rating = float(self.__driver.find_element(By.CSS_SELECTOR, '#boxReview > div > div.col-md-4 > div > div.row.center-vertical > div.col-md-4 > div').text)

                book.price = float(self.__driver.find_element(By.CSS_SELECTOR,
                                                              '#main > div > div > div.bookdetail-row-1.book-detail-block.mt10.mb10 > div > div.col-xs-12.col-md-8.mt10.mb10 > div > div.row > div.col-sm-5.tw-hidden.md\:tw-block > div.price-block.mb15 > p.price').text.replace(
                    ' บาท', ''))
                self.__scraped_selected_books += [book]


            elif book.source == 'ookbee':
                self.__url = book.book_link
                self.__driver.get(self.__url)
                self.__driver.get(self.__url)
                book.name = self.__driver.find_element(By.CSS_SELECTOR,
                                                       '#__next > div > main > div.row.justify-content-center.contentDetailFrame.undefined > div.col-12.col-md > div.contentInfoTitle.d-none.d-md-block > h1').text
                book.author = self.__driver.find_element(By.CSS_SELECTOR,
                                                         '#__next > div > main > div.row.justify-content-center.contentDetailFrame.undefined > div.col-12.col-md > div.row.bibliographyInfo.undefined > div:nth-child(2) > div > div.col-auto.value > a > span').text
                book.publisher = self.__driver.find_element(By.CSS_SELECTOR,
                                                            '#__next > div > main > div.row.justify-content-center.contentDetailFrame.undefined > div.col-12.col-md > div.row.bibliographyInfo.undefined > div:nth-child(1) > div > div.col-auto.value > a > p').text
                book.book_type = self.__driver.find_element(By.CSS_SELECTOR,
                                                            '#__next > div > main > div.row.justify-content-center.contentDetailFrame.undefined > div.col-12.col-md > div.publishInfoFrame > div.publishInfoBody > p:nth-child(4) > span.valueText').text
                book.rating = float(self.__driver.find_element(By.CSS_SELECTOR,
                                                         '#__next > div > main > div.row.justify-content-center.contentDetailFrame.undefined > div.col-12.col-sm-6.col-md-4.col-lg-4 > div.contentRatingInfo > div > p').text.replace('Ratings', ''))*5/100
                book.price = self.__driver.find_element(By.CSS_SELECTOR, '#__next > div > main > div.row.justify-content-center.contentDetailFrame.undefined > div.col-12.col-md > '
                                                                         'div.contentPriceFrame > div.contentPriceFrameBody > div > p').text
                book.price = float(book.price.replace('฿', '').replace('ซื้อฉบับนี้ : ', ''))
                self.__scraped_selected_books += [book]

        return self.__scraped_selected_books


class Facade:
    def __init__(self):
        self.__meb_scraper = MebSearchScraper()
        self.__naiin_scraper = NaiinSearchScraper()
        self.__ookbee_scraper = OokbeeSearchScraper()
        self.__selection_scraper = SelectionScraper()
        self.__book_related_tool = BookRelatedTools()

    def searching(self, selection, keyword):
        if selection == 'meb':
            print('searching meb')
            return self.__meb_scraper.overview_scraping(keyword)
        elif selection == 'naiin':
            print('searching naiin')
            return self.__naiin_scraper.overview_scraping(keyword)
        elif selection == 'ookbee':
            print('searching ookbee')
            return self.__ookbee_scraper.overview_scraping(keyword)
        else:
            print('Invalid selection')

    def scrape_selection(self, book_list):
        return self.__selection_scraper.scrape_book(book_list)

    def books_to_df(self, books):
        return self.__book_related_tool.books_to_df(books)

    def df_to_book(self, df):
        return self.__book_related_tool.df_to_book(df)



